%EDEIR_RUN.M -> MAIN file -> Calibrate EDEIR model to Swiss economy using
%an ALTERNATIVE utility function
%Compute numerically a first-order approximation, 
%second moments, and correlations implied by the 
%Small Open Economy Model With An External Debt-Elastic Interest Rate 
%as presented in chapter 4 of ``Open Economy Macroeconomics,'' 
%by Martin Uribe and Stephanie Schmitt Grohe, Princeton University Press 2017.
%??? Mart???n Uribe and Stephanie  Stephanie Schmitt-Groh???, January 2013.

% Clear workspace & set directory
clear all; close all; fclose('all'); clc;
cd '/MATLAB Drive/Teaching/2021/FS/International Macroeconomics/Problem Set 2/Handout to Students/Question 7'

seed = 1234; %for reproducibility
rng(seed)

% Create the symbolic expressions for the model. 
%You need to run this only once. 
edeir_model_Q7
    %This program generates the files edeir_model_num_eval.m and
    %edeir_model.mat
    
%here, we iterate over parameter values gamma, hbar,
%phi, psi, rho, and etatilde, to match desired moments!

%set starting values! -> similar to reasoning in Section 4.5, Part C 
GAMMA_start=0.5;
HBAR_start = 1;
PSSI_start=0.5;
PHI_start=0.5;
RHO_start=0.5;
ETATILDE_start=0.5;

% % % Targetted moments
% Specify Target Moments "target" (i.e., the values observed in the data 
% for the cyclical component of variables) for:
% second moments (column 1), 
% serial correlation (column 2)
% correlation with GDP (column 3) 
% for the variables y (row 1), c (row 2), i (row 3), h (row 4), 
% tb/y (row 5) --> see Table 4.2 in the textbook 
%
% -> FOR EXAMPLE, target values from Mendoza (Canada 1946-1985)
target = [2.81; 2.46; 9.82; 2.02; 1.87]; %target values for second moments
target(:,2) = [0.62; 0.7; 0.31; 0.54; 0.66]; % targets for serial correlation
target(:,3) = [1; 0.59; 0.64; 0.8; -0.13]; % targets for correlation with output
    %cf. Table 4.2 in the book!
    
runs = 10; %how many different starting values do you want to try?
for iter = 1:runs

    disp(' ')
    disp(strcat(['Running iteration nr. ' num2str(iter)]))
    
    X_start=[GAMMA_start,HBAR_start,PSSI_start,PHI_start,RHO_start,ETATILDE_start]; %starting values
        %inputs are, in this order, GAMMA, HBAR, PSSI, PHI, RHO, ETATILDE
        %importantly, parameters should be strictly positive,
        %and RHO and GAMMA should also be smaller than 1!

    lb=[0.0001, 0.001,0.0001,0.001,0.001,0.001]; %lower bound values for the six parameters 
    ub=[0.99,20,10,10,0.99,10]; %upper bound values for the six parameters

    % Add random change to starting values
    if iter~=1 %first iteration with initial starting values
        random_change = rand(size(X_start)).*3; 
        random_change(1) = rand(1,1);%don't allow too large values for GAMMA
        random_change(5) = rand(1,1);%don't allow too large values for RHO
        X_start = X_start.*random_change;

            %check that all values satisfy the bounds
            for i = 1:length(X_start)
                if X_start(i)<lb(i) 
                    X_start(i)=lb(i);
                elseif X_start(i)>ub(i)
                    X_start(i)=ub(i);
                end
            end
    end
    
    objective=@(X) obj_to_min_Q7(X,target);  %target second moments and serial correlations 
        %function obj_to_min_Q7(X,target) computes SS values for given parameter values,
        %and then verifies whether the specified parameters roughly match the
        %second moments we target --> if the match is not good enough (where good
        %is defined by the "FunctionTolerance" criterion below), it computes a
        %further iteration

    options = optimoptions(@fmincon,'FunctionTolerance', 1e-8,'Display','off'); 
        %specify tolerance value; default is 1e-6
        
    disp(X_start)
        
    try 
        %we use try - catch - end in order to avoid being stuck in a
        %situation where we cannot find an equilibrium!
        %--> i.e., when we do not find an equilibrium with the
        %specified starting values X_start, then we 
        
        PARAMETERS = fmincon(objective,X_start,[],[],[],[],lb,ub,[],options); %optimization

        %extract optimal parameter values
        GAMMA(iter)=PARAMETERS(1);
        HBAR(iter)=PARAMETERS(2);
        PSSI(iter)=PARAMETERS(3);
        PHI(iter)=PARAMETERS(4);
        RHO(iter)=PARAMETERS(5);
        ETATILDE(iter)=PARAMETERS(6);
   
        obj_value(iter) = objective(PARAMETERS);
        
    catch ME
            disp(strcat(['Warning: Starting values ' num2str(X_start) ' yield no Equilibrium']))
            GAMMA(iter)=NaN;
            HBAR(iter)=NaN;
            PSSI(iter)=NaN;
            PHI(iter)=NaN;
            RHO(iter)=NaN;
            ETATILDE(iter)=NaN;
            
            obj_value(iter)=NaN;
            continue
    end
end

% Keep parameter results that minimize objective function
[~, min_param] = min(obj_value); 

GAMMA = GAMMA(min_param);
HBAR = HBAR(min_param);
PSSI = PSSI(min_param);
PHI = PHI(min_param);
RHO = RHO(min_param);
ETATILDE = ETATILDE(min_param);

%recompute SS with optimal parameter values
edeir_ss_Q7;

%Evaluate f and its derivates at the steady state by running edeir_model_num_eval.m
edeir_model_num_eval; 
%this is a .m file produced by running edeir_model.m

%First-stage policy functions
[gx,hx,exitflag]=gx_hx(nfy,nfx,nfyp,nfxp); 
%the function gx_hx.m is available online at http://www.columbia.edu/~mu2166/1st_order/1st_order.htm

%Variance/Covariance matrix of innovation to state vector x_t
varshock = nETASHOCK*nETASHOCK';

%standard deviations
[sigy0,sigx0]=mom(gx,hx,varshock);
stds = sqrt(diag(sigy0));

%correlations with output
corr_xy = sigy0(:,noutput)./stds(noutput)./stds;

%serial correlations
[sigy1,sigx1]=mom(gx,hx,varshock,1);
scorr = diag(sigy1)./diag(sigy0);

%make a table containing second moments
num_table = [stds*100  scorr corr_xy];
        %note how the std. deviations are multiplied by 100 in order to
        %express them in percentage points (see Table 4.2 in book)

%From this table, select variables of interest (output, c, ivv, h, tby, cay)
disp('In This table:');
disp('Rows: y, c, i, h, tb/y, ca/y');
disp('Columns: std,  serial corr., corr with y,');
num_table1 = num_table([noutput nc nivv nh ntby  ncay],:);
disp(num_table1);
disp(' ');
% format long g; %to display more digits after the comma
disp('Optimal Parameters:');
disp('GAMMA HBAR PHI PSSI RHO ETATILDE');
disp([GAMMA HBAR PHI PSSI RHO ETATILDE]);
% format short g;

%Compute Impulse Responses
% -> use the same routine as in Question 6, Problem Set 2

